#
# ExtLinuxConf.py - Simple syslinux config parsing
#
# Copyright 2010 Citrix Systems Ltd.
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys, re, os
import logging
import GrubConf

class ExtLinuxImage(object):
    def __init__(self, lines, path):
        self.reset(lines, path)

    def __repr__(self):
        return ("title: %s\n"
                "  root: %s\n"
                "  kernel: %s\n"
                "  args: %s\n"
                "  initrd: %s\n" %(self.title, self.root, self.kernel,
                                   self.args, self.initrd))
    def reset(self, lines, path):
        self._initrd = self._kernel = self._readonly = None
        self._args = ""
        self.title = ""
        self.lines = []
        self.path = path
        self.root = ""
        map(self.set_from_line, lines)

    def set_from_line(self, line, replace = None):
        (com, arg) = GrubConf.grub_exact_split(line, 2)
        com = com.lower()

        # Special handling for mboot.c32
        if com.lower() == "append" and self.kernel is not None:
            (_,kernel) = self.kernel
            if kernel.endswith("mboot.c32"):
                kernel = None
                args = None
                initrd = None
                modules = arg.split("---")

                if len(modules) == 3: # Assume Xen + Kernel + Initrd
                    (_,kernel,initrd) = modules
                elif len(modules) == 2: # Assume Kernel + Initrd
                    (kernel,initrd) = modules

                if kernel:
                    setattr(self, "kernel", kernel.strip())
                if initrd:
                    setattr(self, "initrd", initrd.strip())

                # Bypass regular self.commands handling
                com = None
            elif "initrd=" in arg:
                # find initrd image in append line
                args = arg.strip().split(" ")
                for a in args:
                    if a.lower().startswith("initrd="):
                        setattr(self, "initrd", a.replace("initrd=", ""))
                        arg = arg.replace(a, "")

        if com is not None and self.commands.has_key(com):
            if self.commands[com] is not None:
                setattr(self, self.commands[com], re.sub('^"(.+)"$', r"\1", arg.strip()))
            else:
                logging.info("Ignored image directive %s" %(com,))
        elif com is not None:
            logging.warning("Unknown image directive %s" %(com,))

        # now put the line in the list of lines
        if replace is None:
            self.lines.append(line)
        else:
            self.lines.pop(replace)
            self.lines.insert(replace, line)

    def set_kernel(self, val):
        if val.find(" ") == -1:
            self._kernel = (None,val)
            self._args = None
            return
        (kernel, args) = val.split(None, 1)
        self._kernel = (None,kernel)
        self._args = args
    def get_kernel(self):
        return self._kernel
    def set_args(self, val):
        self._args = val
    def get_args(self):
        return self._args
    kernel = property(get_kernel, set_kernel)
    args = property(get_args, set_args)

    def set_initrd(self, val):
        self._initrd = (None,val)
    def get_initrd(self):
        return self._initrd
    initrd = property(get_initrd, set_initrd)

    def set_readonly(self, val):
        self._readonly = 1
    def get_readonly(self):
        return self._readonly
    readonly = property(get_readonly, set_readonly)

    # set up command handlers
    commands = {
        "label": "title",
        "kernel": "kernel",
        "append": "args"}

class ExtLinuxConfigFile(object):
    def __init__(self, fn = None):
        self.filename = fn
        self.images = []
        self.timeout = -1
        self._default = 0

        if fn is not None:
            self.parse()

    def new_image(self, title, lines):
        # ExtLinuxImage constructor doesn't have title but since path
        # is being used by get_{kernel|initrd} functions we pass
        # empty string rather than None (see lines above)
        return ExtLinuxImage(lines, "")

    def parse(self, buf = None):
        if buf is None:
            if self.filename is None:
                raise ValueError, "No config file defined to parse!"

            f = open(self.filename, 'r')
            lines = f.readlines()
            f.close()
        else:
            lines = buf.split("\n")

        path = os.path.dirname(self.filename)
        img = []
        for l in lines:
            l = l.strip()
            # skip blank lines
            if len(l) == 0:
                continue
            # skip comments
            if l.startswith('#'):
                continue
            # new image
            if l.lower().startswith("label"):
                if len(img) > 0:
                    self.add_image(ExtLinuxImage(img, path))
                img = [l]
                continue

            if len(img) > 0:
                img.append(l)
                continue

            (com, arg) = GrubConf.grub_exact_split(l, 2)
            com = com.lower()
            if self.commands.has_key(com):
                if self.commands[com] is not None:
                    setattr(self, self.commands[com], arg.strip())
                else:
                    logging.info("Ignored directive %s" %(com,))
            else:
                logging.warning("Unknown directive %s" %(com,))

        if len(img) > 0:
            self.add_image(ExtLinuxImage(img, path))

    def hasPassword(self):
        return False

    def hasPasswordAccess(self):
        return True

    def add_image(self, image):
        self.images.append(image)

    def _get_default(self):
        for i in range(len(self.images)):
            if self.images[i].title == self._default:
                return i
        return 0
    def _set_default(self, val):
        self._default = val
    default = property(_get_default, _set_default)

    commands = { "default": "default",
                 "timeout": "timeout",
                 "serial": None,
                 "prompt": None,
                 "display": None,
                 "f1": None,
                 "f2": None,
                 }
        
if __name__ == "__main__":
    if sys.argv < 2:
        raise RuntimeError, "Need a configuration file to read"
    g = ExtLinuxConfigFile(sys.argv[1])
    for i in g.images:
        print i
    print g.default



#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Copyright (C) XenSource Ltd, 2005
# Author: Dan Smith <danms@us.ibm.com>
# Author: Woody Marvel <marvel@us.ibm.com>
# Author: Ewan Mellor <ewan@xensource.com>

from XmTestLib import *

becomeNonRoot()

status, output = traceCommand("xm help")
eyecatcher = "Usage:"
where = output.find(eyecatcher)
if where == -1:
    FAIL("xm help: didn't see the usage string")



#!/usr/bin/python
                                                                                              
# Copyright (C) International Business Machines Corp., 2005
# Author: Li Ge <lge@us.ibm.com>

# Positive Test: create domain with block attach, attach another, verify both in list


from XmTestLib import *

if ENABLE_HVM_SUPPORT:
    SKIP("Block-list not supported for HVM domains")

config = {"disk":"phy:/dev/ram0,xvda1,w"}
domain = XmTestDomain(extraConfig=config)

try:
    console = domain.start()
except DomainError, e:
    if verbose:
        print e.extra
    FAIL("Unable to create domain")

status, output = traceCommand("xm block-list %s" % domain.getId())
if status != 0:
    FAIL("Fail to list block device")

#Add another virtual block device to the domain
status, output = traceCommand("xm block-attach %s phy:/dev/ram1 xvda2 w" % domain.getId())
if status != 0:
    FAIL("Fail to attach block device")

#Verify block-list on Domain0
status, output = traceCommand("xm block-list %s" % domain.getId())
eyecatcher1 = "51713"
eyecatcher2 = "51714"
where1 = output.find(eyecatcher1)
where2 = output.find(eyecatcher2)
if status != 0:
    FAIL("xm block-list returned bad status, expected 0, status is %i" % status)
elif (where1 < 0) and (where2 < 0):
    FAIL("Fail to list all block devices after attaching another block device")

#Verify attached block device on DomainU
try:
    run = console.runCmd("cat /proc/partitions | grep xvda1;cat /proc/partitions | grep xvda2")
except ConsoleError, e:
    saveLog(console.getHistory())
    FAIL(str(e))

domain.stop()

if run["return"] != 0:
    FAIL("Failed to verify that block dev is attached on DomainU")



#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2004, 2005 Mike Wray <mike.wray@hp.com>
# Copyright (C) 2005 XenSource Ltd
#============================================================================

"""Xend interface to networking control scripts.
"""

import XendOptions
from xen.util import xpopen

def network(op):
    """Call a network control script.

    @param op: operation (start, stop)
    """
    if op not in ['start', 'stop']:
        raise ValueError('Invalid operation: ' + op)
    script = XendOptions.instance().get_network_script()
    if script:
        script.insert(1, op)
        xpopen.call(script)



#!/usr/bin/python

# Copyright (C) International Business Machines Corp., 2005
# Author: Dan Smith <danms@us.ibm.com>

from XmTestLib import *
from XmTestLib.block_utils import *

if ENABLE_HVM_SUPPORT:
    SKIP("Block-detach not supported for HVM domains")

domain = XmTestDomain()

try:
    console = domain.start()
except DomainError, e:
    if verbose:
        print e.extra
    FAIL("Unable to create domain")

block_attach(domain, "phy:/dev/ram0", "xvda1")
try:
    run = console.runCmd("cat /proc/partitions | grep xvda1")
except ConsoleError, e:
    saveLog(console.getHistory())
    FAIL(str(e))

if run["return"] != 0:
    FAIL("Failed to verify that block dev is attached")

block_detach(domain, "xvda1")
try:
    run = console.runCmd("cat /proc/partitions | grep xvda1")
except ConsoleError, e:
    saveLog(console.getHistory())
    FAIL(str(e))

domain.stop()

if run["return"] == 0:
    FAIL("block-detach failed to detach block device")



#!/usr/bin/python
#============================================================================
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#============================================================================
# Copyright (C) 2006 XenSource Ltd.
# Copyright (C) 2006 IBM Corporation
# Copyright (C) 2009 flonatel GmbH & Co. KG
#============================================================================

import atexit
import os
import sys
from XmTestLib import *
from xen.xm import main as xmmain
from xen.xm import XenAPI
from xen.xm.opts import OptionError
from types import DictType
import xml.dom.minidom

sessions=[]

def connect(*args):
    creds = ("", "")
    uri = "http://localhost:9363"

    try:
        session = XenAPI.Session(uri)
    except:
        raise OptionError("Could not create XenAPI session with Xend." \
                          "URI=%s" % uri)
    try:
        session.login_with_password(*creds)
    except:
        raise OptionError("Could not login to Xend. URI=%s" % uri)
    def logout():
        try:
            for s in sessions:
                s.xenapi.session.logout()
        except:
            pass
    sessions.append(session)
    atexit.register(logout)
    return session
